Module: MovableInk::AWS::EC2
- Included in:
- MovableInk::AWS
- Defined in:
- lib/movable_ink/aws/ec2.rb
Instance Method Summary collapse
- #all_instances(region: my_region, no_filter: false) ⇒ Object
- #assign_ip_address(role:) ⇒ Object
- #available_elastic_ips(role:) ⇒ Object
- #default_filter ⇒ Object
- #ec2(region: my_region) ⇒ Object
- #elastic_ips ⇒ Object
- #instance_ip_addresses_by_role(role:, exclude_roles: [], region: my_region, availability_zone: nil, exact_match: false, use_cache: true) ⇒ Object
- #instance_ip_addresses_by_role_ordered(role:, exclude_roles: [], region: my_region, exact_match: false) ⇒ Object
- #instance_tags ⇒ Object
- #instances(role:, exclude_roles: [], region: my_region, availability_zone: nil, exact_match: false, use_cache: true, discovery_type: 'ec2') ⇒ Object
- #instances_with_consul_discovery(role:, region: my_region, availability_zone: nil) ⇒ Object
- #instances_with_ec2_discovery(role:, exclude_roles: [], region: my_region, availability_zone: nil, exact_match: false, use_cache: true) ⇒ Object
- #load_all_instances(region, no_filter: false, filter: nil) ⇒ Object
- #load_mi_env ⇒ Object
- #me ⇒ Object
- #mi_env ⇒ Object
- #mi_env_cache_file_path ⇒ Object
- #private_ip_addresses(instances) ⇒ Object
- #redis_by_role(role, port) ⇒ Object
- #statsd_host ⇒ Object
- #thopter ⇒ Object
- #thopter_instance ⇒ Object
- #unassigned_elastic_ips ⇒ Object
Instance Method Details
#all_instances(region: my_region, no_filter: false) ⇒ Object
41 42 43 44 |
# File 'lib/movable_ink/aws/ec2.rb', line 41 def all_instances(region: my_region, no_filter: false) @all_instances ||= {} @all_instances[region] ||= load_all_instances(region, no_filter: no_filter) end |
#assign_ip_address(role:) ⇒ Object
207 208 209 210 211 212 213 214 |
# File 'lib/movable_ink/aws/ec2.rb', line 207 def assign_ip_address(role:) run_with_backoff do ec2.associate_address({ instance_id: instance_id, allocation_id: available_elastic_ips(role: role).sample.allocation_id }) end end |
#available_elastic_ips(role:) ⇒ Object
203 204 205 |
# File 'lib/movable_ink/aws/ec2.rb', line 203 def available_elastic_ips(role:) unassigned_elastic_ips.select { |address| address..detect { |t| t.key == 'mi:roles' && t.value == role } } end |
#default_filter ⇒ Object
46 47 48 49 50 51 52 53 54 55 |
# File 'lib/movable_ink/aws/ec2.rb', line 46 def default_filter [{ name: 'instance-state-name', values: ['running'] }, { name: 'tag:mi:env', values: [mi_env] }] end |
#ec2(region: my_region) ⇒ Object
7 8 9 10 |
# File 'lib/movable_ink/aws/ec2.rb', line 7 def ec2(region: my_region) @ec2_client ||= {} @ec2_client[region] ||= Aws::EC2::Client.new(region: region) end |
#elastic_ips ⇒ Object
193 194 195 196 197 |
# File 'lib/movable_ink/aws/ec2.rb', line 193 def elastic_ips @all_elastic_ips ||= run_with_backoff do ec2.describe_addresses.addresses end end |
#instance_ip_addresses_by_role(role:, exclude_roles: [], region: my_region, availability_zone: nil, exact_match: false, use_cache: true) ⇒ Object
174 175 176 |
# File 'lib/movable_ink/aws/ec2.rb', line 174 def instance_ip_addresses_by_role(role:, exclude_roles: [], region: my_region, availability_zone: nil, exact_match: false, use_cache: true) private_ip_addresses(instances(role: role, exclude_roles: exclude_roles, region: region, availability_zone: availability_zone, exact_match: exact_match, use_cache: use_cache)) end |
#instance_ip_addresses_by_role_ordered(role:, exclude_roles: [], region: my_region, exact_match: false) ⇒ Object
178 179 180 181 182 183 |
# File 'lib/movable_ink/aws/ec2.rb', line 178 def instance_ip_addresses_by_role_ordered(role:, exclude_roles: [], region: my_region, exact_match: false) instances = instances(role: role, exclude_roles: exclude_roles, region: region, exact_match: exact_match) instances_in_my_az = instances.select { |instance| instance.placement.availability_zone == availability_zone } ordered_instances = instances_in_my_az.shuffle + (instances - instances_in_my_az).shuffle private_ip_addresses(ordered_instances) end |
#instance_tags ⇒ Object
67 68 69 70 71 72 73 |
# File 'lib/movable_ink/aws/ec2.rb', line 67 def @instance_tags ||= run_with_backoff(quiet: true) do ec2.({ filters: [{ name: 'resource-id', values: [instance_id] } ] }). end end |
#instances(role:, exclude_roles: [], region: my_region, availability_zone: nil, exact_match: false, use_cache: true, discovery_type: 'ec2') ⇒ Object
152 153 154 155 156 157 158 159 160 |
# File 'lib/movable_ink/aws/ec2.rb', line 152 def instances(role:, exclude_roles: [], region: my_region, availability_zone: nil, exact_match: false, use_cache: true, discovery_type: 'ec2') if discovery_type == 'ec2' instances_with_ec2_discovery(role: role, exclude_roles: exclude_roles, region: region, availability_zone: availability_zone, exact_match: exact_match, use_cache: use_cache) elsif discovery_type == 'consul' instances_with_consul_discovery(role: role, region: region, availability_zone: availability_zone) else raise MovableInk::AWS::Errors::InvalidDiscoveryTypeError end end |
#instances_with_consul_discovery(role:, region: my_region, availability_zone: nil) ⇒ Object
110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 |
# File 'lib/movable_ink/aws/ec2.rb', line 110 def instances_with_consul_discovery(role:, region: my_region, availability_zone: nil) if role == nil || role == '' raise MovableInk::AWS::Errors::RoleNameRequiredError end = { dc: datacenter(region: region), stale: true, cached: true, passing: true, } [:node_meta] = "availability_zone:#{availability_zone}" unless availability_zone.nil? # We replace underscores with dashes in the role name in order to comply with # consul service naming conventions while still retaining the role name we use # within MI configuration Diplomat::Health.service(role.gsub('_', '-'), ).map { |endpoint| OpenStruct.new ( { private_ip_address: endpoint.Node['Address'], instance_id: endpoint.Node['Meta']['instance_id'], tags: [ { key: 'Name', value: endpoint.Node['Node'] }, { key: 'mi:roles', value: endpoint.Node['Meta']['mi_roles'] }, { key: 'mi:monitoring_roles', value: endpoint.Node['Meta']['mi_monitoring_roles'] } ], placement: { availability_zone: endpoint.Node['Meta']['availability_zone'] } }) } end |
#instances_with_ec2_discovery(role:, exclude_roles: [], region: my_region, availability_zone: nil, exact_match: false, use_cache: true) ⇒ Object
79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 |
# File 'lib/movable_ink/aws/ec2.rb', line 79 def instances_with_ec2_discovery(role:, exclude_roles: [], region: my_region, availability_zone: nil, exact_match: false, use_cache: true) roles = role.split(/\s*,\s*/) if use_cache == false filter = default_filter.push({ name: 'tag:mi:roles', values: roles }) instances = load_all_instances(region, filter: filter) else instances = all_instances(region: region).select { |instance| instance..select{ |tag| tag.key == 'mi:roles' }.detect { |tag| tag_roles = tag.value.split(/\s*,\s*/) if exact_match tag_roles == roles else exclude_roles.push('decommissioned') tag_roles.any? { |tag_role| roles.include?(tag_role) } && !tag_roles.any? { |role| exclude_roles.include?(role) } end } } end if availability_zone instances.select { |instance| instance.placement.availability_zone == availability_zone } else instances end end |
#load_all_instances(region, no_filter: false, filter: nil) ⇒ Object
57 58 59 60 61 62 63 64 65 |
# File 'lib/movable_ink/aws/ec2.rb', line 57 def load_all_instances(region, no_filter: false, filter: nil) filters = no_filter ? nil : (filter || default_filter) run_with_backoff do ec2(region: region).describe_instances(filters: filters).flat_map do |resp| resp.reservations.flat_map(&:instances) end end end |
#load_mi_env ⇒ Object
25 26 27 28 29 30 31 |
# File 'lib/movable_ink/aws/ec2.rb', line 25 def load_mi_env .detect { |tag| tag.key == 'mi:env' } .value rescue NoMethodError raise MovableInk::AWS::Errors::NoEnvironmentTagError end |
#me ⇒ Object
75 76 77 |
# File 'lib/movable_ink/aws/ec2.rb', line 75 def me @me ||= all_instances.select{|instance| instance.instance_id == instance_id}.first rescue nil end |
#mi_env ⇒ Object
16 17 18 19 20 21 22 23 |
# File 'lib/movable_ink/aws/ec2.rb', line 16 def mi_env @mi_env ||= if File.exist?(mi_env_cache_file_path) environments = JSON.parse(File.read(mi_env_cache_file_path)) environments[my_region] || load_mi_env else load_mi_env end end |
#mi_env_cache_file_path ⇒ Object
12 13 14 |
# File 'lib/movable_ink/aws/ec2.rb', line 12 def mi_env_cache_file_path '/etc/movableink/environments.json' end |
#private_ip_addresses(instances) ⇒ Object
170 171 172 |
# File 'lib/movable_ink/aws/ec2.rb', line 170 def private_ip_addresses(instances) instances.map(&:private_ip_address) end |
#redis_by_role(role, port) ⇒ Object
185 186 187 188 189 190 191 |
# File 'lib/movable_ink/aws/ec2.rb', line 185 def redis_by_role(role, port) instance_ip_addresses_by_role(role: role) .shuffle .inject([]) { |redii, instance| redii.push({"host" => instance, "port" => port}) } end |
#statsd_host ⇒ Object
166 167 168 |
# File 'lib/movable_ink/aws/ec2.rb', line 166 def statsd_host instance_ip_addresses_by_role(role: 'statsd', availability_zone: availability_zone, use_cache: false).sample end |
#thopter ⇒ Object
162 163 164 |
# File 'lib/movable_ink/aws/ec2.rb', line 162 def thopter private_ip_addresses(thopter_instance).first end |
#thopter_instance ⇒ Object
33 34 35 36 37 38 39 |
# File 'lib/movable_ink/aws/ec2.rb', line 33 def thopter_instance @thopter_instance ||= all_instances(region: 'us-east-1').select do |instance| instance..select{ |tag| tag.key == 'mi:roles' }.detect do |tag| tag.value.include?('thopter') end end end |
#unassigned_elastic_ips ⇒ Object
199 200 201 |
# File 'lib/movable_ink/aws/ec2.rb', line 199 def unassigned_elastic_ips @unassigned_elastic_ips ||= elastic_ips.select { |address| address.association_id.nil? } end |